class Quandl::Cassandra::Models::Column::Read::Offset < Quandl::Cassandra::Models::Column::Read
  
  def perform
    # skip strategy unless all required attriutes are present
    return unless required_attributes_present?
    # apply strategy
    apply_offset_with_transform
    apply_offset
  end
  
  def required_attributes_present?
    column_ids.present? && column_collapses.present? && columns_trim_start.is_a?(Date) && columns_trim_end.is_a?(Date)
  end
  
  def apply_offset_with_transform
    # rdiff_from needs the data from the current to offset
    if attributes[:row].present? && attributes[:transform] == :rdiff_from
      # limit the results by trim 
      self.trim_start = columns_trim_start.occurrences_of_frequency_ago( attributes[:row], collapse_with_frequency ).end_of_frequency(collapse_with_frequency) if order == :desc
      self.trim_end   = columns_trim_end.occurrences_of_frequency_ahead( attributes[:row], collapse_with_frequency ).end_of_frequency(collapse_with_frequency) if order == :asc
      # the query should not limit or offset the data
      attributes[:limit] = nil
      attributes[:offset] = nil
    end
  end
  
  def apply_offset
    return unless attributes[:offset].present?
    order == :asc ? apply_offset_asc : apply_offset_desc
  end
  
  def apply_offset_asc
    # calculate ranges
    offset_start  = columns_trim_end.occurrences_of_frequency_ahead( offset, collapse_with_frequency ).start_of_frequency(collapse_with_frequency)
    offset_end    = offset_start.occurrences_of_frequency_ahead( accuracy_with_limit, collapse_with_frequency ).end_of_frequency( collapse_with_frequency ) if limit
    # set trims
    self.trim_start = offset_start
    self.trim_end   = offset_end if limit
  end
  
  def apply_offset_desc
    # calculate ranges
    offset_start  = columns_trim_start.occurrences_of_frequency_ago( offset, collapse_with_frequency ).end_of_frequency(collapse_with_frequency)
    offset_end    = offset_start.occurrences_of_frequency_ago( accuracy_with_limit, collapse_with_frequency ).start_of_frequency( collapse_with_frequency ) if limit
    # set trims
    self.trim_start = offset_end  if limit
    self.trim_end   = offset_start
  end
  
  def columns_trim_start
    @trim_start ||= Quandl::Cassandra::Column.find_max_time_by_ids(column_ids, column_collapses, "DESC")
  end
  
  def columns_trim_end
    @trim_end ||= Quandl::Cassandra::Column.find_max_time_by_ids(column_ids, column_collapses, "ASC")
  end
  
  def accuracy_with_limit
    awl = accuracy
    awl = awl + limit - 1 if limit && limit > 0
    awl
  end
  
  def accuracy
    # did the query include an accuracy?
    accuracy = attributes[:accuracy]
    # otherwise guess at the accuracy by collapse
    if accuracy.blank?
      accuracy = case frequency.to_sym
      when :daily     then 0
      when :weekly    then 1
      when :monthly   then 1
      when :quarterly then 1
      when :annual    then 0
      else
        3
      end
    end
    accuracy
  end
  
  def limit
    @limit ||= attributes[:limit]
  end
  
  def offset
    attributes[:offset]
  end
  
  def order
    @order ||= attributes[:order] == :asc ? :asc : :desc
  end
  
  def collapse_with_frequency
    collapse == :source ? frequency : collapse
  end
  
  def frequency
    attributes[:frequency]
  end
  
end