class Quandl::Cassandra::Models::Column::Read::SelectColumns < Quandl::Cassandra::Models::Column::Read
  
  def perform
    # attrs to result hash
    attributes[:data] = count? ? count_data : select_data
  rescue => e
    Quandl::Logger.error("#{attributes} #{e}")
    raise
  end
  
  def count_data
    prepared = Quandl::Cassandra::Base.prepare( statement )
    rows = []
    column_ids.each_with_index do | id, index |
      # pluck column type from collapses
      type = attributes[:column_collapses][index].to_s
      # bind and execute query
      rows << prepared.execute( id, type, :one )
    end
    rows.collect{|r| r.first['count'] }.max
  end
  
  def select_data
    t1 = Time.now
    # fire off the queries
    prepared = Quandl::Cassandra::Base.prepare( statement )
    data = {}
    futures = []
    column_ids.each_with_index do | id, index |
      # pluck column type from collapses
      type = attributes[:column_collapses][index].to_s
      # bind and execute query
      futures << prepared.async.execute( id, type, Quandl::Cassandra::Base.consistency )
    end
    # collect the results
    futures.each_with_index do |future, index|
      # collect result 
      future.value.each do |row|
        data[row['time']] ||= Array.new( column_ids.count )
        data[row['time']][index] ||= row['value']
      end
    end
    Quandl::Logger.debug("(#{t1.elapsed_ms}) #{self.class.name}.select_data")
    data
  end
  
  def statement
    columns = count? ? "COUNT(*)" : "time,value"
    cql = "SELECT #{columns} FROM columns WHERE"
    # cql += "ORDER"
    cql += " time >= #{trim_start.jd} AND " if trim_start?
    cql += " time <= #{trim_end.jd} AND " if trim_end?
    cql += " id = ? AND type = ?"
    cql += " ORDER BY type #{order}"
    cql += " LIMIT #{attributes[:limit]}" if attributes[:limit]
    cql
  end
  
  def order
    @order ||= attributes[:order] == :asc ? :asc : :desc
  end
  
end