class Quandl::Cassandra::Models::Column::Read::Transform < Quandl::Cassandra::Models::Column::Read
  
  def perform
    return unless attributes[:transform].present?
    # apply
    apply_rdiff
    apply_cumul
  end
  
  def apply_cumul
    # rdiff_from needs the data from the current to offset
    return unless transform?( :cumul ) && limit.present?    
    # data table needs to handle the offset since cumul requires all data
    attributes[:data_table] = Quandl::Cassandra::Models::Column::Read::Offset.call(attributes)
    # the query should not limit or offset the data
    attributes[:row] = nil
    attributes[:limit] = nil
    attributes[:offset] = nil
  end
  
  def apply_rdiff
    return unless transform? :rdiff, :diff
    # limit requires limit
    attributes[:limit] = attributes[:limit] + 1 if attributes[:limit]
    # trims should be increased by one
    self.trim_start = trim_start.occurrences_of_frequency_ago( 1, collapse ) if trim_start? && order == :desc
    self.trim_end   = trim_end.occurrences_of_frequency_ahead( 1, collapse ) if trim_end? && order == :asc
  end
  
  def transform?(*keys)
    keys.each do |key|
      return true if key == attributes[:transform]
    end
    false
  end
  
  def limit
    attributes[:limit]
  end
  
  def order
    @order ||= attributes[:order] == :asc ? :asc : :desc
  end
  
end