class Quandl::Cassandra::Models::Column::Read::Type < Quandl::Cassandra::Models::Column::Read
  
  def perform
    # enforce types
    attributes.each do |key, value|
      attributes[key] = enforce_type(key, value)
    end
    # retain pristine copy of attrs
    attributes[:pristine] ||= attributes.clone
  end
  
  def enforce_type(key, value)
    case key
    when *date_types      then parse_date(value)
    when *integer_types   then value.try(:to_i)
    when *sym_types       then value.try(:to_sym)
    when :order           then value.try(:to_sym) == :asc ? :asc : :desc
    else
      value
    end
  end
  
  def sym_types
    [:collapse, :transform, :frequency]
  end
  
  def date_types
    [:trim_start, :trim_end]
  end
  
  def integer_types
    [:limit, :column, :offset, :accuracy, :row]
  end
  
  def parse_date(value)
    date = Date.jd(value.to_i) if value.kind_of?(String) && value.numeric?
    date = Date.jd(value) if value.is_a?(Integer)
    date = Date.parse(value) if value.is_a?(String) && value =~ /^[0-9]{4}\-[0-9]{2}\-[0-9]{2}$/
    date = value if value.is_a?(Date)
    date
  rescue
    nil
  end
  
end