class Quandl::Cassandra::Models::Column::Write < Quandl::Strategy::Strategize
  
  # strategy attributes
  define_attributes :id, :data, :frequency, :column_ids, :frequency_data, :frequency_column_data, :statement_values
  
  require_relative 'write/validate'
  require_relative 'write/insert_columns'
  require_relative 'write/insert_column_attributes'
  require_relative 'write/group_data_by_column'
  require_relative 'write/group_data_by_frequency'
  
  # execute strategy
  def self.perform(attributes)
    # perform strategy
    strategy = Quandl::Strategy.new( attributes ) do |c|
      c.use Quandl::Cassandra::Models::Column::Write::Validate
      c.use Quandl::Cassandra::Models::Column::Write::GroupDataByFrequency
      c.use Quandl::Cassandra::Models::Column::Write::GroupDataByColumn
      c.use Quandl::Cassandra::Models::Column::Write::InsertColumns
      c.use Quandl::Cassandra::Models::Column::Write::InsertColumnAttributes
    end
    strategy.perform
  rescue => e
    Quandl::Logger.error("#{e} #{attributes}")
    raise
  end
  
end