class Quandl::Cassandra::Models::Dataset < Quandl::Cassandra::Base
  
  require_relative 'dataset/columns'
  
  table_name :datasets
  autosave_changes false
  
  define_attributes :id, :data
  
  after_initialize :repair_frequency
  
  before_save :save_data, :save_dataset_attribute
  
  after_save :clear_attributes!
  
  delegate :type, :updated_at, :created_at, :frequency, to: :dataset_attribute, allow_nil: true
  
  include Quandl::Cassandra::Models::Dataset::Columns
  
  def trim_start
    @trim_start ||= Date.jd( data.scoped.limit(1).order(:asc)[0][0] )
  rescue
    nil
  end
  
  def trim_end
    @trim_end ||= Date.jd( data.scoped.limit(1).order(:desc)[0][0] )
  rescue
    nil
  end
  
  def data
    repair_columns
    # data set?
    return read_attribute(:data) if data?
    # read data
    @attributes[:data] ||= data_scope
  end
  
  def data=(rows)
    rows = Quandl::Data.new(rows) unless rows.is_a?(Quandl::Data)
    rows = rows.to_jd
    data_will_change!
    @attributes[:data] = rows
  end
  
  def data_scope
    Quandl::Cassandra::Models::Data.dataset(self)
  end
  
  def dataset_attribute
    @dataset_attribute ||= Quandl::Cassandra::Models::DatasetAttribute.find_or_build(id)
  end
  
  def reload
    clear_attributes!
  end
  
  protected
  
  def save_dataset_attribute
    dataset_attribute.save if dataset_attribute.changed?
  end
  
  def save_data
    Quandl::Cassandra::Models::Column.write( id: id, data: data ) if data_changed?
  end
  
  def clear_attributes!
    super if defined?(super)
    @trim_start = nil
    @trim_end = nil
    @dataset_attribute = nil
    @attributes = { id: id }
  end
  
  def repair_frequency
    return if self.new_record?
    if frequency.blank?
      # repair dataset attribute
      dataset_attribute.frequency = data_scope.limit(100).to_table.frequency.to_s
      dataset_attribute.save
    end
  rescue => e
    Quandl::Logger.error("#{self.class.name}.id #{id} #{e}")
  end
  
  def repair_columns
    return if self.new_record?
    columns.each do |c|
      c.frequency = frequency if c.frequency.blank?
      c.save if c.changed? || c.new_record?
    end
  end
  
end