module Quandl::Cassandra::Models::Dataset::Columns
  
  extend ActiveSupport::Concern
  
  included do
    # define_attributes :column_ids
    before_save :save_columns

  end
  
  module ClassMethods

    def find_column_ids_by_id(id)
      Quandl::Cassandra::Models::Dataset.where( id: id ).pluck(:column_id, :position).sort_by{|r| r[1] }.collect{|r| r[0] }
    end
    
  end
  
  def column_attributes=(column_attrs)
    column_attrs.each_with_index do |attrs, index|
      self.columns[index] ||=  Quandl::Cassandra::Models::Column.new
      self.columns[index].assign_attributes(attrs)
    end
  end

  def column_frequencies
    @column_frequencies ||= columns.collect(&:frequency)
  end

  def column_units
    @column_units ||= columns.collect(&:units)
  end

  def column_names
    @column_names ||= columns.collect(&:name)
  end

  def columns
    return @columns if @columns
    # nothing to do without column ids
    return [] if column_ids.compact.blank?
    # find columns
    columns = Quandl::Cassandra::Models::ColumnAttribute.where( id: column_ids ).all
    # build column where column_id was not found
    @columns = column_ids.collect{|cid| columns.detect{|c| c.id.to_s == cid.to_s } || Quandl::Cassandra::Models::ColumnAttribute.new( id: cid ) }
  end

  def column_ids
    @column_ids ||= id.blank? ? [] : self.class.find_column_ids_by_id(id)
  end


  protected
  
  def clear_attributes!
    @columns = nil
    @column_ids = nil
    @column_frequencies = nil
    @column_names = nil
    @column_units = nil
  end
  
  
  private
  
  def save_columns
    columns.each{|c| c.save if c.changed? || c.new_record? }
  end

end