class Quandl::Cassandra::Models::Multiset < Quandl::Cassandra::Models::Dataset
  
  table_name :datasets
  
  define_attributes :datasets_columns
  
  class << self
    
    def with_columns(datasets_columns)
      self.new( datasets_columns: datasets_columns )
    end
    
  end
  
  def data_scope
    Quandl::Cassandra::Models::Data.collapse(:source).column_ids( column_ids ).column_frequencies( column_frequencies )
  end
  
  def column_frequencies
    @column_frequencies ||= columns.collect(&:frequency)
  end
  
  def column_ids
    @column_ids ||= column_ids_from_datasets_columns
  end
  
  def column_ids_from_datasets_columns
    ids = []
    datasets_columns.to_s.split(',').each do |dataset_column|
      dataset_id, column = dataset_column.split('.')
      datasets[dataset_id] ||= Quandl::Cassandra::Models::Dataset.find(dataset_id).try(:column_ids)
      ids << datasets[dataset_id][ column.to_i - 1 ] if datasets[dataset_id].is_a?(Array)
    end
    ids
  end
  
  def datasets
    @datasets ||= {}
  end
  
  def frequency
    return @frequency if @frequency
    uniq_freqs = column_frequencies.compact.uniq
    Quandl::Operation::Collapse.valid_collapses.each do |collapse|
      @frequency = collapse.to_s if uniq_freqs.include?(collapse.to_s)
    end
    @frequency
  end
  
  def dataset_attribute
    # find or initialize dataset attribute object
    @dataset_attribute ||= Quandl::Cassandra::Models::DatasetAttribute.new
  end
  
end
